/*
 * jack_ringbuffer_vector.h - operations on jack_ringbuffer_data_t[2]
 *
 * Copyright (c) 2006 Dmitry S. Baikov <c0ff@konstruktiv.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef __jack_ringbuffer_vector_h__
#define __jack_ringbuffer_vector_h__

#include <string.h>
#include <jack/ringbuffer.h>

inline
size_t jack_ringbuffer_vector_space(jack_ringbuffer_data_t vec[2])
{
	return vec[0].len + vec[1].len;
}

static
size_t jack_ringbuffer_vector_write(jack_ringbuffer_data_t vec[2], const void *data, size_t size)
{
	size_t left = size;
	const char *in = data;
	while (left && vec[0].len>0) {
		size_t todo = left < vec[0].len ? left : vec[0].len;
		memcpy(vec[0].buf, in, todo);
		in += todo;
		left -= todo;
		vec[0].len -= todo;
		if( vec[0].len <= 0 ) {
			vec[0] = vec[1];
			vec[1].len = 0;
		} else
			vec[0].buf += todo;
	}
	return size - left;
}

static
size_t jack_ringbuffer_vector_read(jack_ringbuffer_data_t vec[2], void *data, size_t size)
{
	size_t left = size;
	char *out = data;
	while (left && vec[0].len>0) {
		size_t todo = left < vec[0].len ? left : vec[0].len;
		memcpy(out, vec[0].buf, todo);
		out += todo;
		left -= todo;
		vec[0].len -= todo;
		if( vec[0].len <= 0 ) {
			vec[0] = vec[1];
			vec[1].len = 0;
		} else
			vec[0].buf += todo;
	}
	return size - left;
}

#endif /* __jack_ringbuffer_vector_h__ */
